@@ -5,7 +5,7 @@
 %  make/0, and to improve usability when using the XPCE profiler and XPCE
 %  graphical debugger
 %
-%  Last updated on August 21, 2019
+%  Last updated on January 28, 2020
 %
 %  This file is part of Logtalk <https://logtalk.org/>
 %  Copyright 1998-2020 Paulo Moura <pmoura@logtalk.org>
@@ -192,6 +192,20 @@ user:prolog_exception_hook(Error, Error, Frame, _CatchFrame) :-
 	'$swi_print_backtrace'(Frames).
 */
 
+
+:- multifile(prolog:debug_control_hook/1).
+
+prolog:debug_control_hook(spy(Module:Entity::Functor/Arity)) :-
+	'$lgt_compile_predicate_indicators'(Functor/Arity, Entity, TFunctor/TArity),
+	'$lgt_user_module_qualification'(TFunctor/TArity, Module:TFunctor/TArity),
+	spy(Module:TFunctor/TArity).
+
+prolog:debug_control_hook(nospy(Module:Entity::Functor/Arity)) :-
+	'$lgt_compile_predicate_indicators'(Functor/Arity, Entity, TFunctor/TArity),
+	'$lgt_user_module_qualification'(TFunctor/TArity, Module:TFunctor/TArity),
+	nospy(Module:TFunctor/TArity).
+
+
 % for e.g. the call stack in the SWI-Prolog graphical tracer
 :- multifile(user:prolog_predicate_name/2).
 
@@ -228,8 +242,8 @@ user:prolog_predicate_name(Goal, Label) :-
 '$lgt_swi_prolog_predicate_name'('$lgt_obj_super_call'(_, _, _), '^^/2 (from obj; same pred)') :- !.
 '$lgt_swi_prolog_predicate_name'('$lgt_ctg_super_call'(_, _, _), '^^/2 (from ctg; same pred)') :- !.
 
-'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _, _), 'call/N') :- !.
-'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _), 'call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _, _, _), 'call/N') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _, _), 'call/1') :- !.
 '$lgt_swi_prolog_predicate_name'('$lgt_quantified_metacall'(_, _, _), 'call/1') :- !.
 '$lgt_swi_prolog_predicate_name'('$lgt_metacall_local'(_, _), 'call/1') :- !.
 '$lgt_swi_prolog_predicate_name'('$lgt_metacall_sender'(_, _, _, _), 'call/1') :- !.
@@ -353,10 +367,7 @@ prolog_clause:unify_clause_hook(Clause, QClause, Module, TermPos0, TermPos) :-
 	logtalk::expand_term((NonTerminal --> GRBody), Clause),
 	'$lgt_swi_prolog_clause:unify_clause_hook'(Clause, (THead :- TBody), Module, TermPos0, TermPos).
 '$lgt_swi_prolog_clause:unify_clause_hook'((Head :- Body), (THead :- TBody), _, TermPos0, TermPos) :-
-	(	'$lgt_swi_unify_clause'((Head :- Body), (THead :- TBody), TermPos0, TermPos) ->
-		true
-	;	writeq('UNIFICATION FAILED'-(THead :- TBody)), nl
-	).
+	'$lgt_swi_unify_clause'((Head :- Body), (THead :- TBody), TermPos0, TermPos).
 '$lgt_swi_prolog_clause:unify_clause_hook'((Head :- Body), THead, _, TermPos0, TermPos) :-
 	'$lgt_swi_unify_clause'((Head :- Body), (THead :- true), TermPos0, TermPos).
 '$lgt_swi_prolog_clause:unify_clause_hook'(Head, THead, _, TermPos0, TermPos) :-
@@ -372,7 +383,22 @@ prolog_clause:make_varnames_hook((Head --> _), (Module:THead :- _), Offsets, Nam
 	atom_concat(CodePrefix, _, TFunctor),
 	N is THeadArity - 1,
 	memberchk(N=EVar, Offsets),
-	Names1 = ['<Entity, Sender, This, Self, MetaVars, CoinductionStack>'=EVar| Names],
+	Names1 = ['c(This, Entity, r(Sender, Self, MetaCallContext, Stack))'=EVar| Names],
+	functor(Head, _, HeadArity),
+	In is HeadArity,
+	memberchk(In=IVar, Offsets),
+	Names2 = ['<DCG_list>'=IVar|Names1],
+	Out is HeadArity + 1,
+	memberchk(Out=OVar, Offsets),
+	Names3 = ['<DCG_tail>'=OVar|Names2],
+	prolog_clause:make_varnames(xx, xx, Offsets, Names3, Bindings).
+prolog_clause:make_varnames_hook((Head --> _), (THead :- _), Offsets, Names, Bindings) :-
+	functor(THead, TFunctor, THeadArity),
+	'$lgt_current_flag_'(code_prefix, CodePrefix),
+	atom_concat(CodePrefix, _, TFunctor),
+	N is THeadArity - 1,
+	memberchk(N=EVar, Offsets),
+	Names1 = ['c(This, Entity, r(Sender, Self, MetaCallContext, Stack))'=EVar| Names],
 	functor(Head, _, HeadArity),
 	In is HeadArity,
 	memberchk(In=IVar, Offsets),
@@ -390,6 +416,14 @@ prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Binding
 	memberchk(N=IVar, Offsets),
 	Names1 = ['c(This, Entity, r(Sender, Self, MetaCallCtx, CoinductionStack))'=IVar| Names],
 	prolog_clause:make_varnames(xx, xx, Offsets, Names1, Bindings).
+prolog_clause:make_varnames_hook(_, (THead :- _), Offsets, Names, Bindings) :-
+	functor(THead, TFunctor, Arity),
+	'$lgt_current_flag_'(code_prefix, CodePrefix),
+	atom_concat(CodePrefix, _, TFunctor),
+	N is Arity - 1,
+	memberchk(N=IVar, Offsets),
+	Names1 = ['c(This, Entity, r(Sender, Self, MetaCallCtx, CoinductionStack))'=IVar| Names],
+	prolog_clause:make_varnames(xx, xx, Offsets, Names1, Bindings).
 
 
 '$lgt_swi_unify_clause'((Head :- Body), (THead :- TBody), TermPos0, TermPos) :-
@@ -430,6 +464,11 @@ prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Binding
 '$lgt_swi_unify_clause_body'(Obj::Msg, _, '$lgt_send_to_obj_ne'(Obj, Msg, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Obj::Msg, _, '$lgt_send_to_obj_ne_'(Obj, Msg, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Obj::Msg, _, '$lgt_send_to_obj_'(Obj, Msg, _), TermPos, TermPos) :- !.
+% implicit message sending via uses/2 directives
+'$lgt_swi_unify_clause_body'(Msg, _, '$lgt_send_to_obj_ne_nv'(_, Msg, _), TermPos, TermPos) :- !.
+'$lgt_swi_unify_clause_body'(Msg, _, '$lgt_send_to_obj_ne'(_, Msg, _), TermPos, TermPos) :- !.
+'$lgt_swi_unify_clause_body'(Msg, _, '$lgt_send_to_obj_ne_'(_, Msg, _), TermPos, TermPos) :- !.
+'$lgt_swi_unify_clause_body'(Msg, _, '$lgt_send_to_obj_'(_, Msg, _), TermPos, TermPos) :- !.
 
 '$lgt_swi_unify_clause_body'(::Msg, _, '$lgt_send_to_self'(Msg, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(::Msg, _, '$lgt_send_to_self_nv'(_, Msg, _), TermPos, TermPos) :- !.
@@ -455,14 +494,14 @@ prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Binding
 	'$lgt_swi_unify_clause_body'(Goal, Entity, TGoal, TermPos0, TermPos1),
 	'$lgt_swi_unify_clause_body'(Recover, Entity, TRecover, TermPos1, TermPos).
 
-'$lgt_swi_unify_clause_body'(CallN, _, '$lgt_metacall'(Closure, ExtraArgs, _), TermPos, TermPos) :- !,
+'$lgt_swi_unify_clause_body'(CallN, _, '$lgt_metacall'(Closure, ExtraArgs, _, _), TermPos, TermPos) :- !,
 	functor(CallN, call, Arity),
 	!,
 	length(ExtraArgs, N),
 	Arity is N + 1,
 	arg(1, CallN, Closure),
 	'$lgt_swi_call_n_args'(ExtraArgs, 2, CallN).
-'$lgt_swi_unify_clause_body'(Goal, _, '$lgt_metacall'(Goal, _), TermPos, TermPos) :- !.
+'$lgt_swi_unify_clause_body'(Goal, _, '$lgt_metacall'(Goal, _, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Goal, _, '$lgt_quantified_metacall'(Goal, _, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Goal, _, '$lgt_metacall_local'(Goal, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Goal, _, '$lgt_metacall_sender'(Goal, _, _, _), TermPos, TermPos) :- !.
@@ -671,8 +710,8 @@ prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Binding
 :- '$set_predicate_attribute'('$lgt_ctg_super_call_'/3, trace, 1).
 :- '$set_predicate_attribute'('$lgt_call_in_this'/2, trace, 1).
 
-:- '$set_predicate_attribute'('$lgt_metacall'/2, trace, 1).
 :- '$set_predicate_attribute'('$lgt_metacall'/3, trace, 1).
+:- '$set_predicate_attribute'('$lgt_metacall'/4, trace, 1).
 :- '$set_predicate_attribute'('$lgt_quantified_metacall'/3, trace, 1).
 :- '$set_predicate_attribute'('$lgt_metacall_local'/2, trace, 1).
 :- '$set_predicate_attribute'('$lgt_metacall_sender'/4, trace, 1).
@@ -809,8 +848,8 @@ prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Binding
 :- meta_predicate '$lgt_call_within_context_nv'(*,*,*).
 :- meta_predicate '$lgt_call_within_context'(*,*,*).
 
-:- meta_predicate '$lgt_metacall'(*,*).
 :- meta_predicate '$lgt_metacall'(*,*,*).
+:- meta_predicate '$lgt_metacall'(*,*,*,*).
 :- meta_predicate '$lgt_quantified_metacall'(*,*,*).
 :- meta_predicate '$lgt_metacall_sender'(*,*,*,*).
 :- meta_predicate '$lgt_metacall_local'(*,*).
