@@ -1,7 +1,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Adapter file for YAP Prolog 6.3.4 and later versions
-%  Last updated on January 6, 2020
+%  Last updated on January 21, 2020
 %
 %  This file is part of Logtalk <https://logtalk.org/>
 %  Copyright 1998-2020 Paulo Moura <pmoura@logtalk.org>
@@ -352,6 +352,7 @@
 '$lgt_default_flag'(duplicated_clauses, silent).
 '$lgt_default_flag'(portability, silent).
 '$lgt_default_flag'(redefined_built_ins, silent).
+'$lgt_default_flag'(redefined_operators, warning).
 '$lgt_default_flag'(deprecated, warning).
 '$lgt_default_flag'(missing_directives, warning).
 '$lgt_default_flag'(duplicated_directives, warning).
@@ -635,6 +636,9 @@
 	% (sloppy replacement for the use_module/1-2 directives)
 	'$lgt_yap_directive_expansion'(use_module(File), Expanded).
 
+'$lgt_yap_directive_expansion'(module(Module,Exports0), [(:- module(Module,Exports))| Clauses]) :-
+	'$lgt_yap_split_predicate_aliases'(Exports0, Exports, Clauses).
+
 '$lgt_yap_directive_expansion'(reexport(File), [(:- use_module(Module, Exports)), (:- export(Exports))]) :-
 	'$lgt_yap_list_of_exports'(File, Module, Exports0),
 	'$lgt_yap_fix_predicate_aliases'(Exports0, Exports).
@@ -748,6 +752,19 @@
 		Imports
 	).
 
+
+'$lgt_yap_split_predicate_aliases'([], [], []).
+'$lgt_yap_split_predicate_aliases'([as(Functor/Arity, Alias)| Exports0], [Alias/Arity| Exports], [Clause| Clauses]) :-
+	!,
+	functor(Template, Functor, Arity),
+	Template =.. [Functor| Arguments],
+	AliasTemplate =.. [Alias| Arguments],
+	Clause = (AliasTemplate :- Template),
+	'$lgt_yap_split_predicate_aliases'(Exports0, Exports, Clauses).
+'$lgt_yap_split_predicate_aliases'([Export| Exports0], [Export| Exports], Clauses) :-
+	'$lgt_yap_split_predicate_aliases'(Exports0, Exports, Clauses).
+
+
 '$lgt_yap_fix_predicate_aliases'([], []).
 '$lgt_yap_fix_predicate_aliases'([Import0| Imports0], [Import| Imports]) :-
 	'$lgt_yap_fix_predicate_aliases_aux'([Import0| Imports0], [Import| Imports]).
